module CP
  # Convienience class for creating boxes.
  class Box < Shape::Poly
    # _w_, and _h_ are dimensions of the box. _offset_ is the offset
    # of the center of gravity.
    def initialize(w, h, density, offset)
      hw = w/2; hh = h/2
      verts = [
               vec2(-hw, -hh),
               vec2(-hw,  hh),
               vec2( hw,  hh),
               vec2( hw, -hh),
              ]
      
      mass = w*h*density
      moment = CP.moment_for_poly(mass, verts, offset)
      
      super(CP::Body.new(mass, moment), verts, offset)
    end
  end

  # Convienience class for creating ball shapes.
  class Ball < Shape::Circle
    # _inner_radius_ only affects the calculation of the moment of
    # inertia. _offset_ is the offset of the center of gravity.
    def initialize(radius, inner_radius, density, offset)
      mass = Math::PI*(radius**2 - inner_radius**2)*density
      moment = CP.moment_for_circle(mass, inner_radius, radius, offset)

      super(CP::Body.new(mass, moment), radius, offset)
    end
  end
  
  class Composite
    attr_reader :body, :shapes
    
    def initialize(*objs)
      @shapes = objs
      @body = CP::Body.new(0.0, 0.0)
      
      mass = 0.0
      moment = 0.0
      @shapes.each do|shape|
        mass += shape.body.m
        moment += shape.body.i
        shape.body = @body
      end
      
      @body.m = mass
      @body.i = moment
    end
  end
  
  class Space
    def add_composite(obj)
      add_body(obj.body)
      obj.shapes.each{|shape| add_shape(shape)}
    end

    # Add a shape and the body it's attached to.
    def add_object(obj)
      case obj
      when CP::Shape
        add_body(obj.body)
        add_shape(obj)
      when CP::Composite
        add_composite(obj)
      end
    end
  end
end
