require 'matrix'

# 2D matrix transformations.
class AS::Transform2D < Matrix
  Identity = Matrix.identity(4)

	# Make a scale transformation.
  def self.scale(x, y)
    AS::Transform2D[
      [x, 0, 0, 0],
      [0, y, 0, 0],
      [0, 0, 1, 0],
      [0, 0, 0, 1],
    ]
  end

	# Make a translate transformation.
  def self.translate(x, y)
    AS::Transform2D[
      [1, 0, 0, x],
      [0, 1, 0, y],
      [0, 0, 1, 0],
      [0, 0, 0, 1],
    ]
  end

	# Make a rotation transformation.
  def self.rotate(a)
    a = a.to_f.deg2rad
    sin = Math.sin(a)
    cos = Math.cos(a)
    
    AS::Transform2D[
      [cos, -sin, 0, 0],
      [sin,  cos, 0, 0],
      [  0,    0, 1, 0],
      [  0,    0, 0, 1],
    ]
  end
  
  def self.ortho(left, right, bottom, top)
		a = 2.0/Float(right - left)
		b = -Float(right + left)/Float(right - left)
		c = 2.0/Float(top - bottom)
		d = -Float(top + bottom)/Float(top - bottom)
		
		AS::Transform2D[
			[a, 0, 0, b],
			[0, c, 0, d],
			[0, 0, 1, 0],
			[0, 0, 0, 1],
		]
  end

	# Scale the transformation.
  def scale(x, y)
    self * AS::Transform2D.scale(x,y)
  end

	# Translate the transformation.
  def translate(x, y)
    self * AS::Transform2D.translate(x,y)
  end

	# Rotate the transformation.
  def rotate(a)
    self * AS::Transform2D.rotate(a)
  end
  
  # Create an AS::PackedMatrix pointing to self.
  def pack
    AS::PackedMatrix.new(self)
  end
end

# A packed representation of an AS::Transform2D or Matrix.
# Used by AS::Scene::MatrixNode
class AS::PackedMatrix
  attr_reader :matrix, :packed_matrix

  def initialize(matrix=AS::Transform2D::Identity)
    @packed_matrix = ""
	
    self.matrix = matrix
  end
  
  def matrix=(matrix)
    raise ArgumentError, "Argument must be a matrix" unless matrix.is_a? Matrix
  
		@matrix = matrix
		@packed_matrix.replace(matrix.transpose.to_a.flatten.pack("f*"))
  end
end