class AS::Menu
	include AS::Gamestate
	
	attr_reader :buttons

	def add_actions
		action(AS::Event.for_key(:Key_ESCAPE, false)){stop}
		action(AS::Event.for_mbutton(1)) do
			mouse_down
		end
		
		action(AS::Event.for_mbutton(1, false)) do
			mouse_up
		end
		
		action(AS::Event.for_mouse) do
			mouse_moved
		end
	end
	
	def get_button_under
		@buttons.find do|button|
			dx = $mx - button.x
			dy = $my - button.y
			
			0 <= dx and dx < button.w and 0 <= dy and dy < button.h
		end
	end

	def initialize
		super
		add_actions
		
		@buttons = []
		@button_under = nil
	end
		
	def mouse_moved
		new = get_button_under
		if new!=@button_under
			@button_under.mouse_exit if @button_under
			new.mouse_enter if new
			@button_under = new
		end
	end
	
	def mouse_up
		@button_under.mouse_up if @button_under
	end
	
	def mouse_down
		@button_under.mouse_down if @button_under
	end
	
	def wakeup_state
		AS::Canvas.set_bounds(0, AS::SCREEN_H, AS::SCREEN_W, 0)
	end

	def update
	end
	
	def draw_background
		AS.clear_screen
	end
	
	def draw
		draw_background
		@buttons.send_each(:draw)
	end
end

class AS::Menu::Button
	attr_reader :x, :y, :w, :h
	
	def initialize(x, y, w, h)
		@x, @y = x, y
		@w, @h = w, h
		@state = false
	end
	
	def mouse_down
	end
	
	def mouse_up
	end
	
	def mouse_enter
		@state = true
	end
	
	def mouse_exit
		@state = false
	end
end

class AS::Menu::ImageButton < AS::Menu::Button
	def initialize(tex, x, y, &block)
		super(x, y, tex.w, tex.h)
		@tex, @block = tex, block
	end
	
	def mouse_up
		@block.call
	end
	
	def draw
		@tex.draw_at(x, y)
	end
end