class Font
	attr_reader :tex
	
	def self.open(filename)
		tex = AS::RectTexture.open(filename + '.png')
		new(tex, filename + '.metrics')
	end

	def initialize(tex, metrics_filename)
		case tex
		when AS::RectTexture: sx, sy = 1, 1
		when AS::Texture: sx, sy = tex.w, tex.h
		else raise 'tex must be an AS::Texture'
		end

		@tex = tex
		load_glyphs(metrics_filename, 1.0/sx, 1.0/sy)
	end
	
	def line2ints(line)
		line.split.map{|str| str.to_i}
	end
	
	def load_glyphs(filename, sx, sy)
		glyphs = {}

		File.open(filename, 'r') do|metrics_file|
			@pad, @line_skip, @ascent, @descent = *line2ints(metrics_file.readline)
			
			for line in metrics_file do
				index, gx, gy, tx, ty, w, h, advance = *line2ints(line)
				
				glyphs[index] = AS::DisplayList.new{
					GL.Begin(GL::QUADS)
						GL.TexCoord((tx   -   @pad)*sx, (ty   -   @pad)*sy)
						GL.Vertex(   gx   -   @pad    ,  gy   -   @pad    )

						GL.TexCoord((tx + w + @pad)*sx, (ty   -   @pad)*sy)
						GL.Vertex(   gx + w + @pad    ,  gy   -   @pad    )

						GL.TexCoord((tx + w + @pad)*sx, (ty + h + @pad)*sy)
						GL.Vertex(   gx + w + @pad    ,  gy + h + @pad    )

						GL.TexCoord((tx   -   @pad)*sx, (ty + h + @pad)*sy)
						GL.Vertex(   gx   -   @pad    ,  gy + h + @pad    )
					GL.End
					
					GL.Translate(advance, 0, 0)
				}
				
				glyphs[?\n] = AS::DisplayList.new{
					GL.PopMatrix
					GL.Translate(0, -@line_skip, 0)
					GL.PushMatrix
				}
				
				glyphs.default = glyphs[?~]
			end
		end
		
		@glyphs = Array.new(256){|i| glyphs[i]}
	end
	
	def render_string(string)
		GL.PushMatrix
			string.each_byte{|char| @glyphs[char].call}
		GL.PopMatrix
	end
end

class TextNode < AS::Scene::TextureNode
	attr_reader :string

	def initialize(font, string)
			@string = string.to_s
			
			rnode = AS::Scene::RubyNode.new(lambda{font.render_string(string)})
		 super(font.tex, rnode)
	end
end